import uuid
from typing import Optional

from sqlmodel import SQLModel, Field
from datetime import datetime, timezone


class BaseModel(SQLModel):
    id: uuid.UUID = Field(default_factory=uuid.uuid4, primary_key=True)
    created_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(timezone.utc)
    )
    updated_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(timezone.utc),
        sa_column_kwargs = {"onupdate": lambda: datetime.now(timezone.utc)}
    )


# App specific models
## Integrations
class IntegrationsModel(BaseModel):
    pass


class IntegrationsOrganizationModel(IntegrationsModel):
    organization: uuid.UUID = Field(index=True)


## Jobs
class JobsModel(BaseModel):
    pass


class JobsOrganizationModel(JobsModel):
    organization: uuid.UUID = Field(index=True)


## Billing
class BillingModel(BaseModel):
    pass


class BillingOrganizationModel(BillingModel):
    organization: uuid.UUID = Field(index=True)


## IAM
class IAMModel(BaseModel):
    pass


class IAMOrganizationModel(BaseModel):
    organization: uuid.UUID = Field(index=True)



import uuid

from sqlalchemy import Column, JSON
from sqlmodel import Field

from ..general.models import JobsOrganizationModel


class Feed(JobsOrganizationModel, table=True):
    name: str = Field(index=True)
    status: str = Field(index=True)


class Job(JobsOrganizationModel, table=True):
    name: str = Field(index=True)
    feed_id: uuid.UUID = Field(foreign_key="feed.id")
    status: str = Field(index=True)
    custom_fields: dict = Field(sa_column=Column(JSON))




import enum
import uuid
from dataclasses import dataclass
from datetime import datetime, timezone
from typing import Optional
from sqlalchemy import Enum as SqlEnum, PrimaryKeyConstraint, Identity, Integer

from sqlalchemy import Column
from sqlmodel import SQLModel, Field

from ..general.models import BillingModel, BillingOrganizationModel


class CurrencyEnum(str, enum.Enum):
    USD = "USD"
    EUR = "EUR"
    GBP = "GBP"
    YPJ = "YPJ"


class BillingEvent(SQLModel, table=True):
    id: Optional[int] = Field(
        default=None,
        sa_column=Column(Integer, Identity(), nullable=False)
    )


    organization: uuid.UUID = Field(index=True)
    event_time: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(timezone.utc),
        nullable=False,
    )
    sku: int = Field(foreign_key="sku.id")

    __table_args__ = (
        PrimaryKeyConstraint("id", "event_time"),
    )


class SKU(SQLModel, table=True):
    id: int = Field(nullable=False, primary_key=True)
    name: str = Field(nullable=False)
    description: str = Field(nullable=False)
    amount: float
    currency: CurrencyEnum = Field(sa_column=Column(SqlEnum(CurrencyEnum)))
    created_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(timezone.utc)
    )
    updated_at: Optional[datetime] = Field(
        default_factory=lambda: datetime.now(timezone.utc),
        sa_column_kwargs={"onupdate": lambda: datetime.now(timezone.utc)}
    )


class BillingProfile(BillingOrganizationModel, table=True):
    stripe_customer_id: str = Field(nullable=False)
